// Copyright 1997-1998 Omni Development, Inc.  All rights reserved.
//
// $Header: /LocalDeveloper/Source/CVS/OmniGroup/OWF/Processors.subproj/OWProcessor.h,v 1.5 1998/01/26 06:11:37 kc Exp $

#import <OmniFoundation/OFObject.h>

@class NSDate, NSLock, NSMutableArray;
@class OFMessageQueue;
@class OWContentType, OWCursor, OWPipeline;

#import <OmniFoundation/OFSimpleLock.h>
#import <OmniFoundation/OFMessageQueuePriorityProtocol.h>

typedef enum {
    OWProcessorStarting, OWProcessorQueued, OWProcessorRunning, OWProcessorAborting, OWProcessorRetired
} OWProcessorStatus;

@interface OWProcessor : OFObject <OFMessageQueuePriority>
{
    OWPipeline *pipeline;

    // For display purposes
    OFSimpleLockType displayablesSimpleLock;
    OWProcessorStatus status;
    NSString *statusString;
    NSDate *firstBytesDate;
    unsigned int bytesProcessed;
    unsigned int totalBytes;
}

+ (NSString *)readableClassName;

+ (OFMessageQueue *)processorQueue;
+ (void)registerProcessorClass:(Class)aClass fromContentType:(OWContentType *)sourceContentType toContentType:(OWContentType *)targetContentType cost:(float)aCost;
+ (void)registerProcessorClass:(Class)aClass fromContentTypeString:(NSString *)sourceContentTypeString toContentTypeString:(NSString *)targetContentTypeString cost:(float)aCost;

- initWithPipeline:(OWPipeline *)aPipeline;

- (OWPipeline *)pipeline;
- (void)nullifyPipeline;

// Processing
- (void)startProcessingInQueue:(OFMessageQueue *)aQueue;
- (void)startProcessing;
- (void)abortProcessing;

// Status
- (void)setStatus:(OWProcessorStatus)newStatus;
- (OWProcessorStatus)status;
- (void)setStatusString:(NSString *)newStatusString;
- (void)setStatusFormat:(NSString *)aFormat, ...;
- (void)setStatusStringWithClassName:(NSString *)newStatus;
- (NSString *)statusString;

- (void)processedBytes:(unsigned int)bytes;
    // Use NSNotFound to indicate an unknown amount.
- (void)processedBytes:(unsigned int)bytes ofBytes:(unsigned int)newTotalBytes;
- (NSDate *)firstBytesDate;
- (unsigned int)bytesProcessed;
- (unsigned int)totalBytes;

@end


@interface OWProcessor (SubclassesOnly)
- (void)processBegin;
- (void)process;
- (void)processEnd;
- (void)processAbort;
@end
