// Copyright 1997-1998 Omni Development, Inc.  All rights reserved.
//
// $Header: /LocalDeveloper/Source/CVS/OmniGroup/OWF/Pipelines.subproj/OWTask.h,v 1.9 1998/03/17 02:10:24 kc Exp $

#import <OmniFoundation/OFObject.h>

@class NSCountedSet, NSDate, NSLock, NSMutableArray;
@class OWContentInfo;

#import <Foundation/NSDate.h> // For NSTimeInterval
#import <OmniFoundation/OFSimpleLock.h>
#import <OmniFoundation/OFMessageQueuePriorityProtocol.h>
#import "OWAddressProtocol.h"
#import "OWPipelineInspectorProtocol.h"

typedef enum {
    PipelineInit, PipelineBuilding, PipelineRunning, PipelinePaused,
    PipelineAborting, PipelineDead
} PipelineState;

@interface OWTask : OFObject
{
    OWContentInfo *contentInfo;
    NSLock *parentContentInfoLock;
    OWContentInfo *parentContentInfo;
    
    NSDate *lastActivationDate;

    struct {
        unsigned int wasActiveOnLastCheck:1;
    } taskFlags;
    PipelineState state;

    id <OWPipelineInspector> inspector;

    OFSimpleLockType displayablesSimpleLock;
    NSString *compositeTypeString;
}

+ (NSString *)HMSStringFromTimeInterval:(NSTimeInterval)interval;

// Init and dealloc
- init;
    // Designated initializer
- initWithName:(NSString *)name contentInfo:(OWContentInfo *)aContentInfo parentContentInfo:(OWContentInfo *)aParentContentInfo;

// Task management
- (void)abortTask;

// Active tree
- (BOOL)treeHasActiveChildren;
- (void)treeActiveStatusMayHaveChanged;
- (void)activateInTree;
- (void)deactivateInTree;
- (void)abortTreeActivity;

// State
- (PipelineState)state;
- (id <OWAddress>)lastAddress;

- (NSTimeInterval)timeSinceTreeActivationInterval;
- (NSTimeInterval)estimatedRemainingTimeInterval;
- (NSTimeInterval)estimatedRemainingTreeTimeInterval;

- (BOOL)hadError;
- (BOOL)isRunning;
- (BOOL)hasThread;
- (NSString *)errorNameString;
- (NSString *)errorReasonString;

- (NSString *)compositeTypeString;
- (BOOL)hasChildrenInCompositeTypeStringsInArray:(NSMutableArray *)compositeTypeStrings andSet:(NSCountedSet *)compositeTypeStringsSet;
- (NSString *)documentsInTreeString;
    // -documentsInTreeString returns NIL if this task is active but no children are -- in that case it's a signal you should use -statusString
- (unsigned int)workDone;
- (unsigned int)workToBeDone;
- (unsigned int)workDoneIfNotFinished;
- (unsigned int)workToBeDoneIfNotFinished;
- (unsigned int)workDoneIncludingChildrenIncludingFinished:(BOOL)includeFinished;
- (unsigned int)workToBeDoneIncludingChildrenIncludingFinished:(BOOL)includeFinished;
- (NSString *)statusString;

// Inspecting
- (void)setInspector:(id <OWPipelineInspector>)newInspector;
- (void)tellInspectorPipelineTreeDidChange;

// Parent contentInfo
- (void)setParentContentInfo:(OWContentInfo *)aParentContentInfo;
- (OWContentInfo *)parentContentInfo;

// ContentInfo
- (OWContentInfo *)contentInfo;
- (void)nullifyContentInfo;

- (void)deepFlushContentCache;
    // Flushes the content cache and all its children

// OFMessageQueue protocol helpers
- (unsigned int)taskPriority;
- (unsigned int)baseTaskPriority; // Without modifiers
- (unsigned int)taskGroup;
- (unsigned int)taskMaximumSimultaneousThreadsInGroup;

- (NSComparisonResult)comparePriority:(OWTask *)aTask;

@end
