// Copyright 1997-1998 Omni Development, Inc.  All rights reserved.
//
// $Header: /LocalDeveloper/Source/CVS/OmniGroup/OmniAppKit/Outline.subproj/OAOutlineDragSupportProtocol.h,v 1.2 1998/01/12 06:52:26 kc Exp $

// This protocol defines the messages someone has to be able to handle for the purposes of dragging (and copying?).

// The outline view parameter in most of these lets the drag controller conceivably handle more than one outline view.

@class NSArray, NSString;
@class NSEvent, NSImage, NSPasteboard, NSView;
@class OAOutlineEntry, OAOutlineView, OAPasteboardHelper;

#import <Foundation/NSGeometry.h> // for NSPoint

@protocol OAOutlineDragSupport

// DRAGGING IN

// tell them what pasteboard types we can accept
- (NSArray *)outlineViewAcceptedPasteboardTypes:(OAOutlineView *)aView;

// given a pasteboard string, the pasteboard type, and a parent entry, create a new entry
- (OAOutlineEntry *)outlineView:(OAOutlineView *)aView entryFromPropertyList:propertyList pasteboardType:(NSString *)type parentEntry:(OAOutlineEntry *)anEntry;

// create a new entry, put an empty bookmark in it, tell it it's parent, and return it
- (OAOutlineEntry *)outlineView:(OAOutlineView *)aView emptyEntryWithParent:(OAOutlineEntry *)anEntry;


// DRAGGING OUT

// return the image to be dragged around as the given entry is dragged
- (NSImage *)outlineView:(OAOutlineView *)aView dragImageForEntry:(OAOutlineEntry *)anEntry;

// tell the pasteboard helper what types we can provide from the given entry
- (void)outlineView:(OAOutlineView *)aView declareTypesForEntry:(OAOutlineEntry *)anEntry pasteboardHelper:(OAPasteboardHelper *)pasteboardHelper;

// start the drag operation
- (void)outlineView:(OAOutlineView *)aView startDragOnEntry:(OAOutlineEntry *)anEntry fromView:(NSView *)view image:(NSImage *)image atPoint:(NSPoint)location event:(NSEvent *)event pasteboardHelper:(OAPasteboardHelper *)pasteboardHelper;

// provide data of the requested type to the pasteboard, for the given entry
- (void)pasteboard:(NSPasteboard *)pasteboard provideData:(OAOutlineEntry *)anEntry forType:(NSString *)aType;

@end
