// Copyright 1997-1998 Omni Development, Inc.  All rights reserved.
//
// $Header: /LocalDeveloper/Source/CVS/OmniGroup/OmniAppKit/Outline.subproj/OAOutlineEntry.h,v 1.2 1998/01/12 06:52:26 kc Exp $

// These are the entries in the outline view. They represent your custom objects.

#import <OmniFoundation/OFObject.h>

@class OAOutlineView;
@class NSArray, NSMutableArray;
@class NSPasteboard;

#import "OAOutlineEntryProtocol.h"
#import "OAOutlineFormatterProtocol.h"

@interface OAOutlineEntry : OFObject <OAOutlineEntry>
{
    OAOutlineEntry *nonretainedParentEntry;
    OAOutlineView *nonretainedOutlineView;
    id representedObject;
    NSMutableArray *childEntries;
    id <OAOutlineFormatter> formatter;
    BOOL hidden;
    float totalHeight;
}

+ (OAOutlineEntry *)entryWithRepresentedObject:(id)anObject inEntry:(OAOutlineEntry *)anEntry;

- initInView:(OAOutlineView *)aView;
- initWithRepresentedObject:(id)anObject inEntry:(OAOutlineEntry *)anEntry;

- (id)representedObject;
    // the object we represent
- (void)setRepresentedObject:(id)anObject;
    // set the represented object

- (OAOutlineView *)outlineView;
    // the OAOutlineView in which we reside
- (void)setOutlineView:(OAOutlineView *)newOutlineView;
    // set the OAOutlineView in which we reside

- (OAOutlineEntry *)parentEntry;
    // our parent entry

- (BOOL)hasChildren;
    // Returns YES if the receiver claims to have child entries.  This should be used rather than [[self childEntries] count] != 0 since the entry might want to lazily fill its child list.

- (NSArray *)childEntries;
    // Returns the array of child entries.  This should be called rather than accessing the ivar directly when you want the children to really be there.  This should not naively be called recursivly since the entry subclass might be creating children lazily for efficiency.

- (NSArray *)allRepresentedObjects;
    // all representedObjects from us, children, grandchildren...

- (BOOL)hidden;
    // hidden == collapsed (closed)
- (void)setHidden:(BOOL)newHidden;
- (void)toggleHidden;

- (void)addDragPoints:(NSMutableArray *)dragPointList for:(OAOutlineEntry *)aDraggedEntry inRect:(NSRect)aRect;
- (void)dragDraw:(NSRect)rect inEntry:(OAOutlineEntry *)anEntry;
- (void)makeEntriesPerformSelector:(SEL)selector withObject:(id)anObject;

- (unsigned)indexOfEntry:(OAOutlineEntry *)anEntry;
- (NSRect)entryRect;
- (float)entryHeight;
- (void)recalculateHeight;

- (void)sortResults;

- (BOOL)promoteEntry:(OAOutlineEntry *)entry;
- (BOOL)demoteEntry:(OAOutlineEntry *)entry;
- (void)insertEntry:(OAOutlineEntry *)entry atIndex:(int)index;
- (void)replaceEntry:(OAOutlineEntry *)entry withEntry:(OAOutlineEntry *)newEntry;
- (void)appendEntry:(OAOutlineEntry *)entry;
- (void)removeEntry:(OAOutlineEntry *)eEntry;
- (void)removeEntryAtIndex:(int)index;
- (void)empty;
- (void)nullifyOutlineView;

- (OAOutlineEntry *)previousEntry;
- (OAOutlineEntry *)nextEntry;
- (OAOutlineEntry *)previousVisibleEntry;
- (OAOutlineEntry *)nextVisibleEntry;
- (OAOutlineEntry *)lastEntry;

- (OAOutlineEntry *)findEntryWithRepresentedObject:anObject;
- (OAOutlineEntry *)entryAndRect:(NSRect *)theRect ofPoint:(NSPoint)point;

- (void)contractAllIncludingSelf:(BOOL)includeSelf;
- (void)expandAllIncludingSelf:(BOOL)includeSelf;

- (void)pasteboard:(NSPasteboard *)pasteboard provideDataForType:(NSString *)type;

// Set the formatter on the OAOutlineView; these methods should only be used by OAOutlineView.
- (id <OAOutlineFormatter>)formatter;
- (void)setFormatter:(id <OAOutlineFormatter>)aFormatter;

@end
