// Copyright 1997-1998 Omni Development, Inc.  All rights reserved.
//
// $Header: /LocalDeveloper/Source/CVS/OmniGroup/OmniAppKit/Preferences.subproj/OAPreferenceClient.h,v 1.4 1998/03/20 05:40:15 kc Exp $

#import <OmniFoundation/OFObject.h>

@class NSDictionary, NSNotification, NSString;
@class NSBox, NSTextField;
@class OFUserDefaults;

#import <AppKit/NSNibDeclarations.h> // For IBOutlet

@interface OAPreferenceClient : OFObject
{
    IBOutlet NSBox *controlBox;

    NSString *title;
    NSDictionary *defaultsDictionary;
    OFUserDefaults *defaults;
}

- initWithTitle:(NSString *)newTitle defaultsDictionary:(NSDictionary *)newDefaults;
    // Creates a new preferences client (with the specified title), which manipulates the specified defaults.

- (NSBox *)controlBox;
    // The controlBox outlet points to the box that will be transferred into the Preferences window when this preference client is selected.

- (void)resetFloatValueToDefaultNamed:(NSString *)defaultName inTextField:(NSTextField *)textField;
- (void)resetIntValueToDefaultNamed:(NSString *)defaultName inTextField:(NSTextField *)textField;


// This is an action, so don't change its type
- restoreDefaults:(id)sender;
    // Restores all defaults for this preference client to their original installation values.

- (void)pickDirectoryForTextField:(NSTextField *)textField;
    // Prompts the user for a directory (using an open panel), then updates the text field to display it and calls -setValueForSender: specifying that field as the sender.

// The default implementation of these methods does nothing:  each subclass is expected to implement them.

- (void)updateUI;
    // This method updates the UI to reflect the current defaults

- (void)setValueForSender:(id)sender;
    // This method updates defaults for a modified UI element (the sender).

// These are public so they can be subclassed.
- (void)valuesHaveChanged;
    // This method should be called whenever a default is changed programmatically.  The default implementation simply calls -updateUI.

- (void)controlTextDidEndEditing:(NSNotification *)notification;
    // The default implementation calls -setValueForSender:, setting the sender to be the notification object (i.e., the text field).

// NSNibAwaking informal protocol
- (void)awakeFromNib;
    // Be sure to call super if you subclass this

@end
