// Copyright 1997 Omni Development, Inc.  All rights reserved.
//
// public $Header: /LocalDeveloper/Source/CVS/OmniGroup/OmniFoundation/DataStructures.subproj/OFByteSet.h,v 1.1 1997/12/19 01:00:55 wjs Exp $

#import "OFObject.h"

@class NSData;

#import <Foundation/NSString.h> /* For unichar */

#define OMNI_BYTE_SET_BITMAP_REP_LENGTH (0xff >> 3)

@interface OFByteSet : OFObject
{
@public
    byte                        bitmapRep[OMNI_BYTE_SET_BITMAP_REP_LENGTH];
}

- (BOOL)byteIsMember:(byte)aByte;
- (void)addByte:(byte)aByte;
- (void)removeByte:(byte)aByte;

- (void)addAllBytes;
- (void)removeAllBytes;

- (void)addBytesFromData:(NSData *)data;
- (void)addBytesFromString:(NSString *)string;
- (void)removeBytesFromData:(NSData *)data;
- (void)removeBytesFromString:(NSString *)string;

- (NSData *)data;
- (NSString *)string;

@end

@interface OFByteSet (PredefinedSets)
+ (OFByteSet *)whitespaceByteSet;
@end

static inline BOOL isByteInByteSet(byte aByte, OFByteSet *byteSet)
{
    return byteSet->bitmapRep[aByte >> 3] & (((unsigned)1) << (aByte & 7));
}

static inline BOOL isCharacterInByteSet(unichar ch, OFByteSet *byteSet)
{
    if (ch & 0xff00)
        return NO;
    return isByteInByteSet(ch, byteSet);
}

static inline void addByteToByteSet(byte aByte, OFByteSet *byteSet)
{
    byteSet->bitmapRep[aByte >> 3] |= (((unsigned)1) << (aByte & 7));
}

static inline void
removeByteFromByteSet(byte aByte, OFByteSet *byteSet)
{
    byteSet->bitmapRep[aByte >> 3] &= ~(((unsigned)1) << (aByte & 7));
}

