// Copyright 1997-1998 Omni Development, Inc.  All rights reserved.
//
// $Header: /Network/Developer/Source/CVS/OmniGroup/OmniHTML/View.subproj/OHHTMLView.h,v 1.16 1998/04/16 23:54:34 wiml Exp $

#import <AppKit/NSView.h>

@class NSAttributedString, NSLayoutManager, NSTextContainer;
@class OWAddress, OHBreak, OHHTMLAnchor, OHHTMLOwner, OHHTMLPageView;
@class OFStaticArray;

#import <AppKit/NSNibDeclarations.h> // For IBAction
#import <AppKit/NSTextView.h> // For NSSelectionGranularity, NSSelectionAffinity
#import <OmniAppKit/OAFindControllerTargetProtocol.h>

#import "OHBasicCell.h"
#import "OHHTMLPageContentProtocol.h"
#import "OHHTMLTextViewWithBackgroundImageProtocol.h"
#import "OHManyToolTipViewProtocol.h"
#import "OHWidthRange.h"

@interface OHHTMLView : NSView <OHHTMLPageContent, OHHTMLTextViewWithBackgroundImage, OHManyToolTipView, OASearchableContent>
{
    OHHTMLOwner *nonretainedHTMLOwner;

    NSTextStorage *textStorage;
    NSLayoutManager *layoutManager;
    NSTextContainer *textContainer;

    NSSize textContainerInset;

    OFStaticArray *paragraphs;
    OFStaticArray *words;
    OFStaticArray *lines;
    OFStaticArray *lays;

    NSMutableArray *marginalCells;
    unsigned int firstUnlaidMarginalCellIndex;

    NSMutableArray *clearBreakObjects;

    NSSelectionGranularity selectionGranularity;
    NSRange selectedRange;

    NSColor *backgroundColor;

    struct {
        unsigned int selectedRangeContainsMultipleFonts:1;
        unsigned int selectedRangeChangedSinceLastFontCalculation:1;
        unsigned int needsRelayout:1;
        unsigned int usesOwnBackgroundColor:1;
        unsigned int cachedWidthRangeIsValid:1;
    } flags;

    float requiredWidth;
    OHWidthRange cachedWidthRange;

    NSPoint draggingStartingLocationInWindow;

    NSArray *trackingArray;
}

// API
- (void)setHTMLOwner:(OHHTMLOwner *)anHTMLOwner;
- (OHHTMLOwner *)htmlOwner;
- (unsigned int)glyphIndexForPoint:(NSPoint)point fractionOfDistanceThroughGlyph:(float *)fraction isReallyOnGlyph:(BOOL *)isReallyOnGlyph;
- (const NSRect *)rectArrayForCharacterRange:(NSRange)characterRange rectCount:(unsigned int *)returnRectCountPtr;
- (const NSRect *)rectArrayForLayRange:(NSRange)layRange rectCount:(unsigned int *)returnRectCountPtr;
- (NSRect)containingRectForCharacterRange:(NSRange)characterRange;
- (NSRect)containingRectForLayRange:(NSRange)layRange;
- (void)setNeedsDisplayForCharacterRange:(NSRange)characterRange;
- (void)scrollCharacterRangeToTop:(NSRange)characterRange;
- (void)scrollCharacterRangeToVisible:(NSRange)characterRange;
- (void)scrollCellToTop:(OHBasicCell *)aCell;
- (void)scrollCellToVisible:(OHBasicCell *)aCell;
- (void)redisplayCell:(OHBasicCell *)aCell;
- (void)relayoutCell:(OHBasicCell *)aCell;

// NSTextView look-alike
- (NSString *)string;
- (void)setBackgroundColor:(NSColor *)aColor;
- (BOOL)drawsBackground;
- (void)setTextContainerInset:(NSSize)inset;
- (NSSize)textContainerInset;
- (NSRange)selectedRange;
- (void)setSelectedRange:(NSRange)charRange;
- (void)setSelectedRange:(NSRange)charRange affinity:(NSSelectionAffinity)affinity stillSelecting:(BOOL)stillSelecting;
- (NSRange)selectionRangeForProposedRange:(NSRange)proposedSelRange granularity:(NSSelectionGranularity)granularity;
- (void)scrollRangeToVisible:(NSRange)range;

// Actions
- (IBAction)relayout:(id)sender;
- (IBAction)showSelectedLink:(id)sender;
- (IBAction)saveSelectedLink:(id)sender;

// Calculating dimensions
- (NSSize)pageSize;

- (OHWidthRange)widthRange;

- (void)setFrameSize:(NSSize)newSize relayout:(BOOL)shouldRelayout;
- (void)setWidth:(unsigned int)newWidth;
- (float)heightForCurrentWidth;

// Building the HTML view
- (void)appendAttributedString:(NSAttributedString *)attributedString;
- (void)setRequiredWidth:(float)aWidth;
- (void)addMarginalCell:(OHBasicCell *)cell;
- (NSArray *)marginalCells;
- (void)setValidMarginalCellCount:(unsigned int)newCount;
- (void)addClearBreakObject:(OHBreak *)breakObject;
- (NSArray *)clearBreakObjects;

// Background
- (BOOL)usesOwnBackgroundColor;
- (void)setUsesOwnBackgroundColor:(BOOL)newSetting;

// Dragging help
- (NSPoint)draggingStartingLocationInWindow;

@end

// Access components of the text system

@interface OHHTMLView (TextSystemOnly)
- (NSTextStorage *)textStorage;
- (NSLayoutManager *)layoutManager;
- (NSTextContainer *)textContainer;
- (OFStaticArray *)paragraphs;
- (OFStaticArray *)words;
- (OFStaticArray *)lines;
- (OFStaticArray *)lays;
- (NSRange)characterRangeForWordRange:(NSRange)wordRange;
- (NSRange)glyphRangeForWordRange:(NSRange)wordRange;
@end

@interface OHHTMLView (SubclassesOnly)
- (void)laidOutContentsAtSize:(NSSize)contentSize;
@end
